#!/usr/bin/perl -w -I/usr/sausalito/perl -I.

use CCE;
use Sauce::Config;
use Sauce::Util;

my $cce = new CCE;
$cce->connectfd(\*STDIN,\*STDOUT);

my $oid = $cce->event_oid();
my $obj = $cce->event_object();

my $device = $obj->{'device'} || 'eth0';
my $ipaddr = $obj->{'ipaddr'};


my $netmask = $obj->{'netmask'} || '255.255.255.255';
my $enabled = $obj->{'enabled'} && $ipaddr;
my $bootproto = $obj->{'bootproto'} ? $obj->{'bootproto'} : 'none';
my ($network, $broadcast) = calcnetwork ($ipaddr, $netmask);
my $onboot = $enabled ? 'yes' : 'no';

if (!$ipaddr) {
	$cce->bye("SUCCESS");
	exit(0);
}

# edit ifcfg file
my $filename = '/etc/sysconfig/network-scripts/ifcfg-' . $device;
my $ok = Sauce::Util::editfile($filename, sub {
	my ($fin, $fout) = (shift, shift);
  print $fout <<EOT ;
# $filename
# generated by $0
#
# ${oid}.device = $device
# ${oid}.ipaddr = $ipaddr
# ${oid}.netmask = $netmask
# ${oid}.enabled = $enabled
DEVICE=$device
BOOTPROTO=$bootproto
ONBOOT=$onboot
BROADCAST=$broadcast
NETWORK=$network
NETMASK=$netmask
IPADDR=$ipaddr
USERCTL=no
EOT
});

my $up = $enabled ? "up" : "down";

# modify the running interface
if ($ipaddr) {
  my @cmd = ('/sbin/ifconfig', $device, 
	$ipaddr, 
	'broadcast', $broadcast,
	'netmask', $netmask,
	$up,
  );
  system(@cmd);
}

chmod(0755,$filename);
$cce->bye('SUCCESS');
exit(0);

####################### helper functions ##############################

sub calcnetwork
{
	my ($ipaddr, $netmask) = (shift, shift);
  my $binip = pack("CCCC", split(/\./, $ipaddr));
  my $binmask = pack("CCCC", split(/\./, $netmask));
  my $binnet = $binip & $binmask;
  my $binbcast = $binnet | ~$binmask;
  my $network = join(".",unpack("CCCC", $binnet));
  my $bcast   = join(".",unpack("CCCC", $binbcast));
  return ($network, $bcast);
}

sub checkquad
{
	my $quad = shift;
  my $tester = join(".",unpack("CCCC",pack("CCCC",split(/\./,$quad))));
  return $quad eq $tester;
}
